defmodule DecentralisedBookIndexWeb.BookLive.Show do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Metadata

  @impl true
  def render(assigns) do
    ~H"""
    <.header>
      {@book.title}
      <:subtitle>{@authors_string}</:subtitle>

      <:actions>
        <.link patch={~p"/books/#{@book}/edit"}>
          <.edit_button>
            Edit
          </.edit_button>
        </.link>
      </:actions>
    </.header>

    <.list>
      <:item title="Id">{@book.id}</:item>

      <:item title="Title">{@book.title}</:item>

      <:item title="Description">{@book.description}</:item>

      <:item title="Published">{@book.published}</:item>

      <:item title="Language">{@book.language}</:item>

      <:item title="Format">{@book.format}</:item>

      <:item title="Page count">{@book.page_count}</:item>

      <:item title="Cover image url">{@book.cover_image_url}</:item>
    </.list>

    <.back navigate={~p"/books"}>Back to books</.back>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok, socket}
  end

  @impl true
  def handle_params(%{"id" => id}, _, socket) do
    book = Ash.get!(Metadata.Book, id, actor: socket.assigns.current_user, load: [:author_roles])
    author_roles = Metadata.get_author_roles_by_book_id!(book.id, load: [:author])
    authors_string = Enum.reduce(
      author_roles, "",
      fn author_role, acc ->
        if acc != "" do
          acc <> ", "
        else
          acc
        end
        <>
        if author_role.role == "" do
          "#{author_role.author.name}"
        else
          "#{author_role.author.name} (#{author_role.role})"
        end
      end
    )

    {:noreply,
     socket
     |> assign(:page_title, page_title(socket.assigns.live_action))
     |> assign(:book, book)
     |> assign(:authors_string, authors_string)
    }
  end

  defp page_title(:show), do: "Show Book"
  defp page_title(:edit), do: "Edit Book"
end
